# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import csv

import datasets


_CITATION = """\
@article{hendryckstest2021,
      title={Measuring Massive Multitask Language Understanding},
      author={Dan Hendrycks and Collin Burns and Steven Basart and Andy Zou and Mantas Mazeika and Dawn Song and Jacob Steinhardt},
      journal={Proceedings of the International Conference on Learning Representations (ICLR)},
      year={2021}
    }
"""

_DESCRIPTION = """\
This is a massive multitask test consisting of multiple-choice questions from various branches of knowledge, covering 57 tasks including elementary mathematics, US history, computer science, law, and more.
"""

_HOMEPAGE = "https://github.com/hendrycks/test"

_URL = "https://huggingface.co/datasets/cais/mmlu/resolve/main/data.tar"

_SUBJECTS = [
    "all",
    "abstract_algebra",
    "anatomy",
    "astronomy",
    "business_ethics",
    "clinical_knowledge",
    "college_biology",
    "college_chemistry",
    "college_computer_science",
    "college_mathematics",
    "college_medicine",
    "college_physics",
    "computer_security",
    "conceptual_physics",
    "econometrics",
    "electrical_engineering",
    "elementary_mathematics",
    "formal_logic",
    "global_facts",
    "high_school_biology",
    "high_school_chemistry",
    "high_school_computer_science",
    "high_school_european_history",
    "high_school_geography",
    "high_school_government_and_politics",
    "high_school_macroeconomics",
    "high_school_mathematics",
    "high_school_microeconomics",
    "high_school_physics",
    "high_school_psychology",
    "high_school_statistics",
    "high_school_us_history",
    "high_school_world_history",
    "human_aging",
    "human_sexuality",
    "international_law",
    "jurisprudence",
    "logical_fallacies",
    "machine_learning",
    "management",
    "marketing",
    "medical_genetics",
    "miscellaneous",
    "moral_disputes",
    "moral_scenarios",
    "nutrition",
    "philosophy",
    "prehistory",
    "professional_accounting",
    "professional_law",
    "professional_medicine",
    "professional_psychology",
    "public_relations",
    "security_studies",
    "sociology",
    "us_foreign_policy",
    "virology",
    "world_religions",
]


class Mmlu(datasets.GeneratorBasedBuilder):
    """Measuring Massive Multitask Language Understanding, consisting of 57 tasks"""

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name=sub, version=datasets.Version("1.0.0"), description=f"MMLU Subject {sub}"
        )
        for sub in _SUBJECTS
    ]

    def _info(self):
        features = datasets.Features(
            {
                "question": datasets.Value("string"),
                "subject": datasets.Value("string"),
                "choices": datasets.features.Sequence(datasets.Value("string")),
                "answer": datasets.features.ClassLabel(num_classes=4, names=["A", "B", "C", "D"]),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        archive = dl_manager.download(_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split("auxiliary_train"),
                gen_kwargs={
                    "iter_archive": dl_manager.iter_archive(archive),
                    "split": "auxiliary_train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"iter_archive": dl_manager.iter_archive(archive), "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "iter_archive": dl_manager.iter_archive(archive),
                    "split": "val",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split("dev"),
                gen_kwargs={
                    "iter_archive": dl_manager.iter_archive(archive),
                    "split": "dev",
                },
            ),
        ]

    def _generate_examples(self, iter_archive, split):
        """Yields examples as (key, example) tuples."""
        n_yielded_files = 0
        for id_file, (path, file) in enumerate(iter_archive):
            if f"data/{split}/" in path:
                if split == "auxiliary_train" or f"{self.config.name}_{split}.csv" in path or self.config.name == "all":
                    subset = path.split("/")[-1].rsplit("_",1)[0] if split != "auxiliary_train" else ""
                    n_yielded_files += 1
                    lines = (line.decode("utf-8") for line in file)
                    reader = csv.reader(lines)
                    for id_line, data in enumerate(reader):
                        yield f"{id_file}_{id_line}", {"question": data[0], "choices": data[1:5], "answer": data[5], "subject": subset}
                    if (n_yielded_files == 8 or split != "auxiliary_train") and self.config.name != "all":
                        break



{"content-type": "text/plain; charset=utf-8", "content-length": "2345", "connection": "keep-alive", "date": "Tue, 24 Oct 2023 11:08:39 GMT", "x-powered-by": "huggingface-moon", "x-request-id": "Root=1-6537a5b7-75ee90ab09df1c611066e609;7a291474-324f-4645-a0ab-69392158ebbc", "access-control-allow-origin": "https://huggingface.co", "vary": "Origin", "access-control-expose-headers": "X-Repo-Commit,X-Request-Id,X-Error-Code,X-Error-Message,ETag,Link,Accept-Ranges,Content-Range", "x-repo-commit": "cfe0ba3f6d5ad4e689e77fd6b081a7df70222ba2", "accept-ranges": "bytes", "content-security-policy": "default-src none; sandbox", "etag": "\"f8fe2cb434cefda404c506d541959e2fefc86884\"", "x-cache": "Miss from cloudfront", "via": "1.1 9d2f0fb353e3a984b1bc4b73796eed40.cloudfront.net (CloudFront)", "x-amz-cf-pop": "HAN50-P2", "x-amz-cf-id": "Ao_AqHUeGKa5U4mYHx4jcFQvJaAdMLb-v6NJr3Z7umGLPLE5ahqCLQ=="}


{"content-type": "text/plain; charset=utf-8", "content-length": "5844", "connection": "keep-alive", "date": "Tue, 24 Oct 2023 11:21:17 GMT", "x-powered-by": "huggingface-moon", "x-request-id": "Root=1-6537a8ad-7daf5d854a54b4bf2d935eb1;73562070-3041-4616-8842-c658e72ec5b5", "access-control-allow-origin": "https://huggingface.co", "vary": "Origin", "access-control-expose-headers": "X-Repo-Commit,X-Request-Id,X-Error-Code,X-Error-Message,ETag,Link,Accept-Ranges,Content-Range", "x-repo-commit": "7a00892cd331d78a88c8c869d0224a5cdd149848", "accept-ranges": "bytes", "content-security-policy": "default-src none; sandbox", "etag": "\"1ab4dfa5484df579fa5fc4e1848afe9112ecf66f\"", "x-cache": "Miss from cloudfront", "via": "1.1 d9afbb25777f4a3d5e4f6ee69980e69e.cloudfront.net (CloudFront)", "x-amz-cf-pop": "HKG62-C2", "x-amz-cf-id": "VhJ1jR1J44my6nZyIT8YEFA_BSbfJnxlA9ZGVD3wPeKPoEUfBadesQ=="}


{"content-type": "text/plain; charset=utf-8", "content-length": "1924", "connection": "keep-alive", "date": "Tue, 24 Oct 2023 11:08:50 GMT", "x-powered-by": "huggingface-moon", "x-request-id": "Root=1-6537a5c2-1a8036d901b0c1546ff96865;5b2b9943-2d48-4de2-9d15-b1d7968f1987", "access-control-allow-origin": "https://huggingface.co", "vary": "Origin", "access-control-expose-headers": "X-Repo-Commit,X-Request-Id,X-Error-Code,X-Error-Message,ETag,Link,Accept-Ranges,Content-Range", "x-repo-commit": "cfe0ba3f6d5ad4e689e77fd6b081a7df70222ba2", "accept-ranges": "bytes", "content-security-policy": "default-src none; sandbox", "etag": "\"d9cee703ae23284f63078d8a15aca6d7c5614fdc\"", "x-cache": "Miss from cloudfront", "via": "1.1 b5c536696b4eaa5e71ecb973177c119a.cloudfront.net (CloudFront)", "x-amz-cf-pop": "HAN50-P2", "x-amz-cf-id": "Kb-_csqjJLSYGnru1agUzwpD4poxeqUM_9B3NmMJe24XdcrC5sgPPg=="}


from torch.utils import cpp_extension
import pathlib
import os
import subprocess

def _get_cuda_bare_metal_version(cuda_dir):
    raw_output = subprocess.check_output([cuda_dir + "/bin/nvcc", "-V"],
                                         universal_newlines=True)
    output = raw_output.split()
    release_idx = output.index("release") + 1
    release = output[release_idx].split(".")
    bare_metal_major = release[0]
    bare_metal_minor = release[1][0]

    return raw_output, bare_metal_major, bare_metal_minor

def _create_build_dir(buildpath):
    try:
        os.mkdir(buildpath)
    except OSError:
        if not os.path.isdir(buildpath):
            print(f"Creation of the build directory {buildpath} failed")

# Check if cuda 11 is installed for compute capability 8.0
cc_flag = []
_, bare_metal_major, bare_metal_minor = _get_cuda_bare_metal_version(cpp_extension.CUDA_HOME)
if int(bare_metal_major) >= 11:
    cc_flag.append('-gencode')
    cc_flag.append('arch=compute_80,code=sm_80')
    if int(bare_metal_minor) >= 7:
        cc_flag.append('-gencode')
        cc_flag.append('arch=compute_90,code=sm_90')

# Build path
srcpath = pathlib.Path(__file__).parent.absolute()
buildpath = srcpath / 'build'
_create_build_dir(buildpath)

def _cpp_extention_load_helper(name, sources, extra_cuda_flags):
    return cpp_extension.load(
        name=name,
        sources=sources,
        build_directory=buildpath,
        extra_cflags=['-O3', ],
        extra_cuda_cflags=['-O3',
                           '-gencode', 'arch=compute_70,code=sm_70',
                           '--use_fast_math'] + extra_cuda_flags + cc_flag,
        verbose=1
    )

extra_flags = []

cache_autogptq_cuda_256_sources = ["./cache_autogptq_cuda_256.cpp",
           "./cache_autogptq_cuda_kernel_256.cu"]
cache_autogptq_cuda_256 = _cpp_extention_load_helper("cache_autogptq_cuda_256", cache_autogptq_cuda_256_sources, extra_flags)



# Copyright (c) Alibaba Cloud.
#
# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

from transformers import PretrainedConfig


class QWenConfig(PretrainedConfig):
    model_type = "qwen"
    keys_to_ignore_at_inference = ["past_key_values"]

    def __init__(
        self,
        vocab_size=151936,
        hidden_size=4096,
        num_hidden_layers=32,
        num_attention_heads=32,
        emb_dropout_prob=0.0,
        attn_dropout_prob=0.0,
        layer_norm_epsilon=1e-6,
        initializer_range=0.02,
        max_position_embeddings=8192,
        scale_attn_weights=True,
        use_cache=True,
        bf16=False,
        fp16=False,
        fp32=False,
        kv_channels=128,
        rotary_pct=1.0,
        rotary_emb_base=10000,
        use_dynamic_ntk=True,
        use_logn_attn=True,
        use_flash_attn="auto",
        intermediate_size=22016,
        no_bias=True,
        tie_word_embeddings=False,
        use_cache_quantization=False,
        use_cache_kernel=False,
        softmax_in_fp32=False,
        **kwargs,
    ):
        self.vocab_size = vocab_size
        self.hidden_size = hidden_size
        self.intermediate_size = intermediate_size
        self.num_hidden_layers = num_hidden_layers
        self.num_attention_heads = num_attention_heads
        self.emb_dropout_prob = emb_dropout_prob
        self.attn_dropout_prob = attn_dropout_prob
        self.layer_norm_epsilon = layer_norm_epsilon
        self.initializer_range = initializer_range
        self.scale_attn_weights = scale_attn_weights
        self.use_cache = use_cache
        self.max_position_embeddings = max_position_embeddings
        self.bf16 = bf16
        self.fp16 = fp16
        self.fp32 = fp32
        self.kv_channels = kv_channels
        self.rotary_pct = rotary_pct
        self.rotary_emb_base = rotary_emb_base
        self.use_dynamic_ntk = use_dynamic_ntk
        self.use_logn_attn = use_logn_attn
        self.use_flash_attn = use_flash_attn
        self.no_bias = no_bias
        self.use_cache_quantization = use_cache_quantization
        self.use_cache_kernel = use_cache_kernel
        self.softmax_in_fp32 = softmax_in_fp32
        super().__init__(
            tie_word_embeddings=tie_word_embeddings,
            **kwargs
        )



from typing import Any, Dict

import torch
from transformers import AutoModelForCausalLM, AutoTokenizer

from peft import PeftConfig, PeftModel


class EndpointHandler:
    def __init__(self, path=""):
        # load model and processor from path
        self.tokenizer = AutoTokenizer.from_pretrained(path)
        try:
            config = PeftConfig.from_pretrained(path)
            model = AutoModelForCausalLM.from_pretrained(
                config.base_model_name_or_path,
                return_dict=True,
                load_in_8bit=True,
                device_map="auto",
                torch_dtype=torch.float16,
                trust_remote_code=True,
            )
            model.resize_token_embeddings(len(self.tokenizer))
            model = PeftModel.from_pretrained(model, path)
        except Exception:
            model = AutoModelForCausalLM.from_pretrained(
                path, device_map="auto", load_in_8bit=True, torch_dtype=torch.float16, trust_remote_code=True
            )
        self.model = model
        self.device = "cuda" if torch.cuda.is_available() else "cpu"

    def __call__(self, data: Dict[str, Any]) -> Dict[str, str]:
        # process input
        inputs = data.pop("inputs", data)
        parameters = data.pop("parameters", None)

        # preprocess
        inputs = self.tokenizer(inputs, return_tensors="pt").to(self.device)

        # pass inputs with all kwargs in data
        if parameters is not None:
            outputs = self.model.generate(**inputs, **parameters)
        else:
            outputs = self.model.generate(**inputs)

        # postprocess the prediction
        prediction = self.tokenizer.decode(outputs[0], skip_special_tokens=True)

        return [{"generated_text": prediction}]



import datetime
import json
import os
import argparse
import tempfile
import shutil
from typing import Annotated, Union
from fastapi import FastAPI, Header, Request
from fastapi.responses import StreamingResponse, Response
import httpx
from pydantic import BaseSettings
import pytz
from olah.files import file_get_generator, file_head_generator
from olah.lfs import lfs_get_generator

from olah.meta import check_commit_hf, meta_generator

app = FastAPI(debug=False)

class AppSettings(BaseSettings):
    # The address of the model controller.
    repos_path: str = "./repos"
    hf_url: str = "https://huggingface.co"
    hf_lfs_url: str = "https://cdn-lfs.huggingface.co"
    mirror_url: str = "http://localhost:8090"
    mirror_lfs_url: str = "http://localhost:8090"


@app.get("/api/{repo_type}s/{org}/{repo}/revision/{commit}")
async def meta_proxy(repo_type: str, org: str, repo: str, commit: str, request: Request):
    if not await check_commit_hf(app, repo_type, org, repo, commit):
        return Response(status_code=404)
    generator = meta_generator(app, repo_type, org, repo, commit, request)
    headers = await generator.__anext__()
    return StreamingResponse(generator, headers=headers)

@app.head("/{repo_type}s/{org}/{repo}/resolve/{commit}/{file_path:path}")
@app.head("/{org}/{repo}/resolve/{commit}/{file_path:path}")
async def file_head_proxy(org: str, repo: str, commit: str, file_path: str, request: Request, repo_type: str = "model"):
    if not await check_commit_hf(app, repo_type, org, repo, commit):
        return Response(status_code=404)
    generator = file_head_generator(app, repo_type, org, repo, commit, file_path, request)
    headers = await generator.__anext__()
    return StreamingResponse(generator, headers=headers)

@app.get("/{repo_type}s/{org}/{repo}/resolve/{commit}/{file_path:path}")
@app.get("/{org}/{repo}/resolve/{commit}/{file_path:path}")
async def file_proxy(org: str, repo: str, commit: str, file_path: str, request: Request, repo_type: str = "model"):
    if not await check_commit_hf(app, repo_type, org, repo, commit):
        return Response(status_code=404)
    generator = file_get_generator(app, repo_type, org, repo, commit, file_path, request)
    headers = await generator.__anext__()
    return StreamingResponse(generator, headers=headers)

@app.get("/repos/{dir1}/{dir2}/{hash_repo}/{hash_file}")
async def lfs_proxy(dir1: str, dir2: str, hash_repo: str, hash_file: str, request: Request):
    repo_type = "model"
    lfs_url = f"{app.app_settings.hf_lfs_url}/repos/{dir1}/{dir2}/{hash_repo}/{hash_file}"
    save_path = f"{dir1}/{dir2}/{hash_repo}/{hash_file}"
    generator = lfs_get_generator(app, repo_type, lfs_url, save_path, request)
    headers = await generator.__anext__()
    return StreamingResponse(generator, headers=headers)

@app.get("/datasets/hendrycks_test/{hash_file}")
async def lfs_proxy(hash_file: str, request: Request):
    repo_type = "dataset"
    lfs_url = f"{app.app_settings.hf_lfs_url}/datasets/hendrycks_test/{hash_file}"
    save_path = f"hendrycks_test/{hash_file}"
    generator = lfs_get_generator(app, repo_type, lfs_url, save_path, request)
    headers = await generator.__anext__()
    return StreamingResponse(generator, headers=headers)

if __name__ in ["__main__", "olah.server"]:
    parser = argparse.ArgumentParser(
        description="Olah Huggingface Mirror Server."
    )
    parser.add_argument("--host", type=str, default="localhost")
    parser.add_argument("--port", type=int, default=8090)
    parser.add_argument("--repos-path", type=str, default="./repos")
    parser.add_argument("--hf-url", type=str, default="https://huggingface.co")
    parser.add_argument("--hf-lfs-url", type=str, default="https://cdn-lfs.huggingface.co")
    parser.add_argument("--mirror-url", type=str, default="http://localhost:8090")
    parser.add_argument("--mirror-lfs-url", type=str, default="http://localhost:8090")
    args = parser.parse_args()

    app.app_settings = AppSettings(repos_path=args.repos_path)

    import uvicorn
    if __name__ == "__main__":
        uvicorn.run("olah.server:app", host=args.host, port=args.port)






import os
import shutil
import tempfile
from typing import Literal
from fastapi import Request

import httpx

from olah.constants import CHUNK_SIZE, WORKER_API_TIMEOUT

async def check_commit_hf(app, repo_type: Literal["model", "dataset"], org: str, model: str, commit: str) -> bool:
    async with httpx.AsyncClient() as client:
        response = await client.get(f"{app.app_settings.hf_url}/api/{repo_type}s/{org}/{model}/revision/{commit}",
                   timeout=WORKER_API_TIMEOUT)
    return response.status_code == 200

async def meta_generator(app, repo_type: Literal["model", "dataset"], org: str, model: str, commit: str, request: Request):
    headers = {k: v for k, v in request.headers.items()}
    headers.pop("host")

    # save
    repos_path = app.app_settings.repos_path
    save_dir = os.path.join(repos_path, f"api/{repo_type}s/{org}/{model}/revision/{commit}")
    save_path = os.path.join(save_dir, "meta.json")
    if not os.path.exists(save_dir):
        os.makedirs(save_dir, exist_ok=True)
    
    use_cache = os.path.exists(save_path)
    # proxy
    if use_cache:
        yield request.headers
        with open(save_path, "rb") as f:
            while True:
                chunk = f.read(CHUNK_SIZE)
                if not chunk:
                    break
                yield chunk
    else:
        try:
            temp_file_path = None
            async with httpx.AsyncClient() as client:
                with tempfile.NamedTemporaryFile(mode="wb", delete=False) as temp_file:
                    async with client.stream(
                        method="GET", url=f"{app.app_settings.hf_url}/api/{repo_type}s/{org}/{model}/revision/{commit}",
                        headers=headers,
                        timeout=WORKER_API_TIMEOUT,
                    ) as response:
                        response_headers = response.headers
                        yield response_headers

                        async for raw_chunk in response.aiter_raw():
                            if not raw_chunk:
                                continue
                            temp_file.write(raw_chunk)
                            yield raw_chunk
                    temp_file_path = temp_file.name
                    
                shutil.copyfile(temp_file_path, save_path)
        finally:
            if temp_file_path is not None:
                os.remove(temp_file_path)



{"content-type": "text/plain; charset=utf-8", "content-length": "1770", "connection": "keep-alive", "date": "Tue, 24 Oct 2023 11:28:45 GMT", "x-powered-by": "huggingface-moon", "x-request-id": "Root=1-6537aa6d-559c0323777b087b12becd84;9b0c0eb5-5301-429e-98dd-0d539b31afea", "access-control-allow-origin": "https://huggingface.co", "vary": "Origin", "access-control-expose-headers": "X-Repo-Commit,X-Request-Id,X-Error-Code,X-Error-Message,ETag,Link,Accept-Ranges,Content-Range", "x-repo-commit": "b1bcda690655777373f57ea6614eb095ec2c886f", "accept-ranges": "bytes", "content-security-policy": "default-src none; sandbox", "etag": "\"b48aaf777a3bff767ce11413fc96b10e15c8b313\"", "x-cache": "Miss from cloudfront", "via": "1.1 94db66e0b13cce35d9db0c342571f976.cloudfront.net (CloudFront)", "x-amz-cf-pop": "HKG62-C2", "x-amz-cf-id": "n6R6t2SoprBkDNNGpQKM0NhumJQ3Jx9bbZm9nJPHwvFIo2NwKtfujQ=="}


{"content-type": "text/plain; charset=utf-8", "content-length": "5666", "connection": "keep-alive", "date": "Tue, 24 Oct 2023 11:21:12 GMT", "x-powered-by": "huggingface-moon", "x-request-id": "Root=1-6537a8a8-59ed8ebb086db1b93c087c05;287be3e9-8757-4ea7-bba5-e7f354456e21", "access-control-allow-origin": "https://huggingface.co", "vary": "Origin", "access-control-expose-headers": "X-Repo-Commit,X-Request-Id,X-Error-Code,X-Error-Message,ETag,Link,Accept-Ranges,Content-Range", "x-repo-commit": "7a00892cd331d78a88c8c869d0224a5cdd149848", "accept-ranges": "bytes", "content-security-policy": "default-src none; sandbox", "etag": "\"dbc9739b9e15d692ab2cc749c1d650df087b5ba4\"", "x-cache": "Miss from cloudfront", "via": "1.1 1969b693060171ec284d3d0f19afb534.cloudfront.net (CloudFront)", "x-amz-cf-pop": "HKG62-C2", "x-amz-cf-id": "mBDdSxv-zczpQwp59iExCSX19J0PNs2a1I5Bn2hyh5K-2PwOvLiaJA=="}